@@ -128,12 +128,6 @@ private:
     ConversionSegments m_segments;
     int                m_start_id;    // number of commited segments
     int                m_cur_segment; // relative position from m_start_id
-
-    // colors
-    unsigned int       m_conversion_fg_color;
-    unsigned int       m_conversion_bg_color;
-    unsigned int       m_selected_segment_fg_color;
-    unsigned int       m_selected_segment_bg_color;
 };
 
 }
