@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) Hiroyuki Ikezoe <poincare@ikezoe.net>
- *  Copyright (C) 2004 Takuro Ashie <ashie@homa.ne.jp>
+ *  Copyright (C) 2004 - 2005 Hiroyuki Ikezoe <poincare@ikezoe.net>
+ *  Copyright (C) 2004 - 2005 Takuro Ashie <ashie@homa.ne.jp>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -79,7 +79,7 @@ AnthyInstance::AnthyInstance (AnthyFactory   *factory,
     : IMEngineInstanceBase     (factory, encoding, id),
       m_factory                (factory),
       m_on_init                (true),
-      m_preedit                (*this, m_key2kana_tables),
+      m_preedit                (*this),
       m_preedit_string_visible (false),
       m_lookup_table_visible   (false),
       m_n_conv_key_pressed     (0),
@@ -102,7 +102,7 @@ AnthyInstance::~AnthyInstance ()
 bool
 AnthyInstance::is_nicola_thumb_shift_key (const KeyEvent &key)
 {
-    if (m_preedit.get_typing_method () != SCIM_ANTHY_TYPING_METHOD_NICOLA)
+    if (get_typing_method () != SCIM_ANTHY_TYPING_METHOD_NICOLA)
         return false;
 
     if (util_match_key_event (m_factory->m_left_thumb_keys, key, 0xFFFF) ||
@@ -130,8 +130,8 @@ AnthyInstance::process_key_event_input (const KeyEvent &key)
 
         if (need_commit) {
             if (is_realtime_conversion () &&
-                m_preedit.get_input_mode () != SCIM_ANTHY_MODE_LATIN &&
-                m_preedit.get_input_mode () != SCIM_ANTHY_MODE_WIDE_LATIN)
+                get_input_mode () != SCIM_ANTHY_MODE_LATIN &&
+                get_input_mode () != SCIM_ANTHY_MODE_WIDE_LATIN)
             {
                 m_preedit.convert (SCIM_ANTHY_CANDIDATE_NORMAL,
                                    is_single_segment ());
@@ -194,7 +194,7 @@ AnthyInstance::process_key_event (const KeyEvent& key)
 
     // FIXME!
     // for NICOLA thumb shift key
-    if (m_preedit.get_typing_method () == SCIM_ANTHY_TYPING_METHOD_NICOLA &&
+    if (get_typing_method () == SCIM_ANTHY_TYPING_METHOD_NICOLA &&
         is_nicola_thumb_shift_key (key))
     {
         if (process_key_event_input (key))
@@ -221,7 +221,7 @@ AnthyInstance::process_key_event (const KeyEvent& key)
         return false;
 
     // for input key event
-    if (m_preedit.get_typing_method () != SCIM_ANTHY_TYPING_METHOD_NICOLA ||
+    if (get_typing_method () != SCIM_ANTHY_TYPING_METHOD_NICOLA ||
         !is_nicola_thumb_shift_key (key))
     {
         if (process_key_event_input (key))
@@ -566,11 +566,11 @@ AnthyInstance::install_properties (void)
         }
     }
 
-    set_input_mode(m_preedit.get_input_mode ());
+    set_input_mode(get_input_mode ());
     set_conversion_mode (m_conv_mode);
-    set_typing_method (m_preedit.get_typing_method ());
-    set_period_style (m_key2kana_tables.get_period_style (),
-                      m_key2kana_tables.get_comma_style ());
+    set_typing_method (get_typing_method ());
+    set_period_style (m_preedit.get_period_style (),
+                      m_preedit.get_comma_style ());
 
     register_properties (m_properties);
 }
@@ -610,7 +610,7 @@ AnthyInstance::set_input_mode (InputMode mode)
         }
     }
 
-    if (mode != m_preedit.get_input_mode ()) {
+    if (mode != get_input_mode ()) {
         m_preedit.set_input_mode (mode);
         set_preedition ();
     }
@@ -680,14 +680,13 @@ AnthyInstance::set_typing_method (TypingMethod method)
         }
     }
 
-    if (method != m_preedit.get_typing_method ()) {
+    if (method != get_typing_method ()) {
         Key2KanaTable *fundamental_table = NULL;
         if (method == SCIM_ANTHY_TYPING_METHOD_ROMAJI)
             fundamental_table = m_factory->m_custom_romaji_table;
         else if (method == SCIM_ANTHY_TYPING_METHOD_KANA)
             fundamental_table = m_factory->m_custom_kana_table;
-        m_preedit.set_typing_method
-            (method, fundamental_table);
+        m_preedit.set_typing_method (method);
     }
 }
 
@@ -735,10 +734,10 @@ AnthyInstance::set_period_style (PeriodStyle period,
         }
     }
 
-    if (period != m_key2kana_tables.get_period_style ())
-        m_key2kana_tables.set_period_style (period);
-    if (comma != m_key2kana_tables.get_comma_style ())
-        m_key2kana_tables.set_comma_style (comma);
+    if (period != m_preedit.get_period_style ())
+        m_preedit.set_period_style (period);
+    if (comma != m_preedit.get_comma_style ())
+        m_preedit.set_comma_style (comma);
 }
 
 bool
@@ -802,6 +801,16 @@ AnthyInstance::action_revert (void)
     return true;
 }
 
+bool
+AnthyInstance::action_cancel_all (void)
+{
+    if (!m_preedit.is_preediting ())
+        return false;
+
+    reset ();
+    return true;
+}
+
 bool
 AnthyInstance::action_commit (bool learn)
 {
@@ -899,7 +908,7 @@ AnthyInstance::action_insert_space (void)
     bool is_wide = false;
 
     if (m_factory->m_space_type == "FollowMode") {
-        InputMode mode = m_preedit.get_input_mode ();
+        InputMode mode = get_input_mode ();
         if (mode == SCIM_ANTHY_MODE_LATIN ||
             mode == SCIM_ANTHY_MODE_HALF_KATAKANA)
         {
@@ -928,7 +937,7 @@ AnthyInstance::action_insert_alternative_space (void)
     bool is_wide = false;
 
     if (m_factory->m_space_type == "FollowMode") {
-        InputMode mode = m_preedit.get_input_mode ();
+        InputMode mode = get_input_mode ();
         if (mode == SCIM_ANTHY_MODE_LATIN ||
             mode == SCIM_ANTHY_MODE_HALF_KATAKANA)
         {
@@ -1231,8 +1240,7 @@ AnthyInstance::action_select_next_candidate (void)
         set_lookup_table ();
 
     int end = m_lookup_table.number_of_candidates () - 1;
-    if (m_lookup_table.get_cursor_pos () == end)
-    {
+    if (m_lookup_table.get_cursor_pos () == end) {
         m_lookup_table.set_cursor_pos (0);
     } else {
         m_lookup_table.cursor_down ();
@@ -1298,6 +1306,7 @@ AnthyInstance::action_candidates_page_up(void)
 {
     if (!m_preedit.is_converting ()) return false;
     if (!is_selecting_candidates ()) return false;
+    if (!m_lookup_table_visible) return false;
 
     m_lookup_table.page_up ();
 
@@ -1312,6 +1321,7 @@ AnthyInstance::action_candidates_page_down (void)
 {
     if (!m_preedit.is_converting ()) return false;
     if (!is_selecting_candidates ()) return false;
+    if (!m_lookup_table_visible) return false;
 
     m_lookup_table.page_down ();
 
@@ -1429,7 +1439,7 @@ AnthyInstance::action_circle_typing_method (void)
 {
     TypingMethod method;
 
-    method = m_preedit.get_typing_method ();
+    method = get_typing_method ();
     if (method == SCIM_ANTHY_TYPING_METHOD_NICOLA)
         method = SCIM_ANTHY_TYPING_METHOD_ROMAJI;
     else if (method == SCIM_ANTHY_TYPING_METHOD_KANA)
@@ -1447,12 +1457,12 @@ AnthyInstance::action_circle_kana_mode (void)
 {
     InputMode mode;
 
-    if (m_preedit.get_input_mode () == SCIM_ANTHY_MODE_LATIN ||
-        m_preedit.get_input_mode () == SCIM_ANTHY_MODE_WIDE_LATIN)
+    if (get_input_mode () == SCIM_ANTHY_MODE_LATIN ||
+        get_input_mode () == SCIM_ANTHY_MODE_WIDE_LATIN)
     {
         mode = SCIM_ANTHY_MODE_HIRAGANA;
     } else {
-        switch (m_preedit.get_input_mode ()) {
+        switch (get_input_mode ()) {
         case SCIM_ANTHY_MODE_HIRAGANA:
             mode = SCIM_ANTHY_MODE_KATAKANA;
             break;
@@ -1474,13 +1484,13 @@ AnthyInstance::action_circle_kana_mode (void)
 bool
 AnthyInstance::action_toggle_latin_mode (void)
 {
-    if (m_preedit.get_input_mode () == SCIM_ANTHY_MODE_LATIN ||
-        m_preedit.get_input_mode () == SCIM_ANTHY_MODE_WIDE_LATIN)
+    if (get_input_mode () == SCIM_ANTHY_MODE_LATIN ||
+        get_input_mode () == SCIM_ANTHY_MODE_WIDE_LATIN)
     {
         set_input_mode (m_prev_input_mode);
         m_preedit.set_input_mode (m_prev_input_mode);
     } else {
-        m_prev_input_mode = m_preedit.get_input_mode ();
+        m_prev_input_mode = get_input_mode ();
         set_input_mode (SCIM_ANTHY_MODE_LATIN);
         m_preedit.set_input_mode (SCIM_ANTHY_MODE_LATIN);
     }
@@ -1491,12 +1501,12 @@ AnthyInstance::action_toggle_latin_mode (void)
 bool
 AnthyInstance::action_toggle_wide_latin_mode (void)
 {
-    if (m_preedit.get_input_mode () == SCIM_ANTHY_MODE_LATIN ||
-        m_preedit.get_input_mode () == SCIM_ANTHY_MODE_WIDE_LATIN)
+    if (get_input_mode () == SCIM_ANTHY_MODE_LATIN ||
+        get_input_mode () == SCIM_ANTHY_MODE_WIDE_LATIN)
     {
         set_input_mode (m_prev_input_mode);
     } else {
-        m_prev_input_mode = m_preedit.get_input_mode ();
+        m_prev_input_mode = get_input_mode ();
         set_input_mode (SCIM_ANTHY_MODE_WIDE_LATIN);
     }
 
@@ -1580,6 +1590,102 @@ AnthyInstance::action_convert_to_wide_latin (void)
     return convert_kana (SCIM_ANTHY_CANDIDATE_WIDE_LATIN);
 }
 
+bool
+AnthyInstance::action_convert_char_type_forward (void)
+{
+    if (!m_preedit.is_preediting ())
+        return false;
+
+    unset_lookup_table ();
+
+    if (m_preedit.is_converting ()) {
+        int idx = m_preedit.get_selected_segment ();
+        if (idx < 0) {
+            action_revert ();
+            m_preedit.finish ();
+            m_preedit.convert (SCIM_ANTHY_CANDIDATE_HIRAGANA, true);
+        } else {
+            int cand = m_preedit.get_selected_candidate ();
+            switch (cand)
+            {
+            case SCIM_ANTHY_CANDIDATE_HIRAGANA:
+                m_preedit.select_candidate (SCIM_ANTHY_CANDIDATE_KATAKANA);
+                break;
+            case SCIM_ANTHY_CANDIDATE_KATAKANA:
+                m_preedit.select_candidate (SCIM_ANTHY_CANDIDATE_HALF_KATAKANA);
+                break;
+            case SCIM_ANTHY_CANDIDATE_HALF_KATAKANA:
+                m_preedit.select_candidate (SCIM_ANTHY_CANDIDATE_WIDE_LATIN);
+                break;
+            case SCIM_ANTHY_CANDIDATE_WIDE_LATIN:
+                m_preedit.select_candidate (SCIM_ANTHY_CANDIDATE_LATIN);
+                break;
+            case SCIM_ANTHY_CANDIDATE_LATIN:
+                m_preedit.select_candidate (SCIM_ANTHY_CANDIDATE_HIRAGANA);
+                break;
+            default:
+                m_preedit.select_candidate (SCIM_ANTHY_CANDIDATE_HIRAGANA);
+                break;
+            }
+        }
+    } else {
+        m_preedit.finish ();
+        m_preedit.convert (SCIM_ANTHY_CANDIDATE_HIRAGANA, true);
+    }
+
+    set_preedition ();
+
+    return true;
+}
+
+bool
+AnthyInstance::action_convert_char_type_backward (void)
+{
+    if (!m_preedit.is_preediting ())
+        return false;
+
+    unset_lookup_table ();
+
+    if (m_preedit.is_converting ()) {
+        int idx = m_preedit.get_selected_segment ();
+        if (idx < 0) {
+            action_revert ();
+            m_preedit.finish ();
+            m_preedit.convert (SCIM_ANTHY_CANDIDATE_HIRAGANA, true);
+        } else {
+            int cand = m_preedit.get_selected_candidate ();
+            switch (cand)
+            {
+            case SCIM_ANTHY_CANDIDATE_HIRAGANA:
+                m_preedit.select_candidate (SCIM_ANTHY_CANDIDATE_LATIN);
+                break;
+            case SCIM_ANTHY_CANDIDATE_KATAKANA:
+                m_preedit.select_candidate (SCIM_ANTHY_CANDIDATE_HIRAGANA);
+                break;
+            case SCIM_ANTHY_CANDIDATE_HALF_KATAKANA:
+                m_preedit.select_candidate (SCIM_ANTHY_CANDIDATE_KATAKANA);
+                break;
+            case SCIM_ANTHY_CANDIDATE_WIDE_LATIN:
+                m_preedit.select_candidate (SCIM_ANTHY_CANDIDATE_HALF_KATAKANA);
+                break;
+            case SCIM_ANTHY_CANDIDATE_LATIN:
+                m_preedit.select_candidate (SCIM_ANTHY_CANDIDATE_WIDE_LATIN);
+                break;
+            default:
+                m_preedit.select_candidate (SCIM_ANTHY_CANDIDATE_HIRAGANA);
+                break;
+            }
+        }
+    } else {
+        m_preedit.finish ();
+        m_preedit.convert (SCIM_ANTHY_CANDIDATE_HIRAGANA, true);
+    }
+
+    set_preedition ();
+
+    return true;
+}
+
 bool
 AnthyInstance::action_add_word (void)
 {
@@ -1603,6 +1709,24 @@ AnthyInstance::action_regist_word (void)
 }
 #endif
 
+AnthyFactory *
+AnthyInstance::get_factory (void)
+{
+    return m_factory;
+}
+
+TypingMethod
+AnthyInstance::get_typing_method (void)
+{
+    return m_preedit.get_typing_method ();
+}
+
+InputMode
+AnthyInstance::get_input_mode (void)
+{
+    return m_preedit.get_input_mode ();
+}
+
 void
 AnthyInstance::trigger_property (const String &property)
 {
@@ -1667,9 +1791,8 @@ void
 AnthyInstance::reload_config (const ConfigPointer &config)
 {
     // set romaji settings
-    m_key2kana_tables.set_symbol_width (m_factory->m_romaji_half_symbol);
-    m_key2kana_tables.set_number_width (m_factory->m_romaji_half_number);
-    m_preedit.set_allow_split_romaji (m_factory->m_romaji_allow_split);
+    m_preedit.set_symbol_width (m_factory->m_romaji_half_symbol);
+    m_preedit.set_number_width (m_factory->m_romaji_half_number);
 
     // set input mode
     if (m_on_init || !m_factory->m_show_input_mode_label) {
@@ -1688,28 +1811,13 @@ AnthyInstance::reload_config (const ConfigPointer &config)
     // set typing method
     if (m_on_init || !m_factory->m_show_typing_method_label) {
         if (m_factory->m_typing_method == "NICOLA")
-            m_preedit.set_typing_method
-                (SCIM_ANTHY_TYPING_METHOD_NICOLA, NULL);
+            m_preedit.set_typing_method (SCIM_ANTHY_TYPING_METHOD_NICOLA);
         else if (m_factory->m_typing_method == "Kana")
-            m_preedit.set_typing_method
-                (SCIM_ANTHY_TYPING_METHOD_KANA,
-                 m_factory->m_custom_kana_table);
+            m_preedit.set_typing_method (SCIM_ANTHY_TYPING_METHOD_KANA);
         else
-            m_preedit.set_typing_method
-                (SCIM_ANTHY_TYPING_METHOD_ROMAJI,
-                 m_factory->m_custom_romaji_table);
+            m_preedit.set_typing_method (SCIM_ANTHY_TYPING_METHOD_ROMAJI);
     } else {
-        if (m_preedit.get_typing_method () == SCIM_ANTHY_TYPING_METHOD_NICOLA)
-            m_preedit.set_typing_method
-                (SCIM_ANTHY_TYPING_METHOD_NICOLA, NULL);
-        else if (m_preedit.get_typing_method () == SCIM_ANTHY_TYPING_METHOD_KANA)
-            m_preedit.set_typing_method
-                (SCIM_ANTHY_TYPING_METHOD_KANA,
-                 m_factory->m_custom_kana_table);
-        else
-            m_preedit.set_typing_method
-                (SCIM_ANTHY_TYPING_METHOD_ROMAJI,
-                 m_factory->m_custom_romaji_table);
+        m_preedit.set_typing_method (m_preedit.get_typing_method ());
     }
 
     // set conversion mode
@@ -1727,39 +1835,23 @@ AnthyInstance::reload_config (const ConfigPointer &config)
     // set period style
     if (m_on_init || !m_factory->m_show_period_style_label) {
         if (m_factory->m_period_style == "WideLatin") {
-            m_key2kana_tables.set_comma_style  (SCIM_ANTHY_COMMA_WIDE);
-            m_key2kana_tables.set_period_style (SCIM_ANTHY_PERIOD_WIDE);
+            m_preedit.set_comma_style  (SCIM_ANTHY_COMMA_WIDE);
+            m_preedit.set_period_style (SCIM_ANTHY_PERIOD_WIDE);
         } else if (m_factory->m_period_style == "Latin") {
-            m_key2kana_tables.set_comma_style  (SCIM_ANTHY_COMMA_HALF);
-            m_key2kana_tables.set_period_style (SCIM_ANTHY_PERIOD_HALF);
+            m_preedit.set_comma_style  (SCIM_ANTHY_COMMA_HALF);
+            m_preedit.set_period_style (SCIM_ANTHY_PERIOD_HALF);
         } else if (m_factory->m_period_style == "Japanese") {
-            m_key2kana_tables.set_comma_style  (SCIM_ANTHY_COMMA_JAPANESE);
-            m_key2kana_tables.set_period_style (SCIM_ANTHY_PERIOD_JAPANESE);
+            m_preedit.set_comma_style  (SCIM_ANTHY_COMMA_JAPANESE);
+            m_preedit.set_period_style (SCIM_ANTHY_PERIOD_JAPANESE);
         } else if (m_factory->m_period_style == "WideLatin_Japanese") {
-            m_key2kana_tables.set_comma_style  (SCIM_ANTHY_COMMA_WIDE);
-            m_key2kana_tables.set_period_style (SCIM_ANTHY_PERIOD_JAPANESE);
+            m_preedit.set_comma_style  (SCIM_ANTHY_COMMA_WIDE);
+            m_preedit.set_period_style (SCIM_ANTHY_PERIOD_JAPANESE);
         } else {
-            m_key2kana_tables.set_comma_style  (SCIM_ANTHY_COMMA_JAPANESE);
-            m_key2kana_tables.set_period_style (SCIM_ANTHY_PERIOD_JAPANESE);
+            m_preedit.set_comma_style  (SCIM_ANTHY_COMMA_JAPANESE);
+            m_preedit.set_period_style (SCIM_ANTHY_PERIOD_JAPANESE);
         }
     }
 
-    // set ten key type
-    if (m_factory->m_ten_key_type == "Half")
-        m_preedit.set_ten_key_type (SCIM_ANTHY_TEN_KEY_HALF);
-    else if (m_factory->m_ten_key_type == "Wide")
-        m_preedit.set_ten_key_type (SCIM_ANTHY_TEN_KEY_WIDE);
-    else 
-        m_preedit.set_ten_key_type (SCIM_ANTHY_TEN_KEY_FOLLOW_MODE);
-
-    // set auto convert
-    if (m_factory->m_behavior_on_period == "Convert")
-        m_preedit.set_behavior_on_period (SCIM_ANTHY_CONVERT_ON_PERIOD);
-    else if (m_factory->m_behavior_on_period == "Commit")
-        m_preedit.set_behavior_on_period (SCIM_ANTHY_COMMIT_ON_PERIOD);
-    else
-        m_preedit.set_behavior_on_period (SCIM_ANTHY_NONE_ON_PERIOD);
-
     // set lookup table
     if (m_factory->m_cand_win_page_size > 0)
         m_lookup_table.set_page_size (m_factory->m_cand_win_page_size);
