@@ -885,7 +885,7 @@ StringConfigData config_keyboards_converting [] =
         SCIM_ANTHY_CONFIG_CONV_CHAR_TYPE_FORWARD_KEY_DEFAULT,
         SCIM_ANTHY_CONFIG_CONV_CHAR_TYPE_FORWARD_KEY_DEFAULT,
         N_("Convert character type to forward"),
-        N_("Select keys to convert characgter type to forward"),
+        N_("Select keys to convert character type to forward"),
         N_("Rotate character type forward."),
         NULL,
         false,
@@ -895,7 +895,7 @@ StringConfigData config_keyboards_converting [] =
         SCIM_ANTHY_CONFIG_CONV_CHAR_TYPE_BACKWARD_KEY_DEFAULT,
         SCIM_ANTHY_CONFIG_CONV_CHAR_TYPE_BACKWARD_KEY_DEFAULT,
         N_("Convert character type to backward"),
-        N_("Select keys to convert characgter type to backward"),
+        N_("Select keys to convert character type to backward"),
         N_("Rotate character type backward."),
         NULL,
         false,
